# This Makefile is used as an interface between the Peachtree make-driven
# build system and the ghc Cabal-driven package system.  It presumes the
# existence of several environment variables.

# The ghc executable to use during package builds
GHC = $(SYSROOT)$(BINDIR)/ghc

# Get the version of ghc in sysroot
GHC_VER = $(shell env SYSROOT=$(SYSROOT) LD_LIBRARY_PATH=$(SYSROOT)$(LIBDIR) $(SYSROOT)$(LIBDIR)/ghc/bin/ghc --version | rev | cut -d ' ' -f 1 | rev)

# We need to set up a couple environment variables to trick ghc into
# running against a sysroot.  It really doesn't want to do that.
export GCONV_PATH := $(WRKSYSROOT)$(LIBDIR)/gconv

# Extend CONFIGURE_OPTS now that we have GHC_VER and PKG defined.
CONFIGURE_OPTS += --libsubdir='ghc-$(GHC_VER)/$(PKG)-$(VER)' \
                  --datasubdir=$(PKG) \
                  --extra-include-dirs=$(WRKSYSROOT)$(X11_INCLUDEDIR) \
                  --extra-lib-dirs=$(WRKSYSROOT)$(X11_LIBDIR) \

all: build

mangle-package-cache:
	haveconf="$$(ls -1 $(SYSROOT)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/*conf 2>/dev/null)" ; \
	if [ ! -z "$${haveconf}" ]; then \
		for f in $(SYSROOT)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/*conf; do \
			sed -i -e "s|-dirs: $(LIBDIR)|-dirs: $(SYSROOT)$(LIBDIR)|" $${f} ; \
		done ; \
		env SYSROOT=$(SYSROOT) LD_LIBRARY_PATH=$(SYSROOT)$(LIBDIR) $(SYSROOT)$(BINDIR)/ghc-pkg recache ; \
	fi

unmangle-package-cache:
	haveconf="$$(ls -1 $(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/*conf 2>/dev/null)" ; \
	if [ ! -z "$${haveconf}" ]; then \
		for f in $(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/*conf; do \
			sed -i -e 's|\"-Wl,--sysroot=.\+||' -e "s|$(SYSROOT)||" $${f} ; \
		done ; \
	fi

build: mangle-package-cache
ifeq ($(HS_MODULES),)
	env $(CONFIGURE_ENV) runhaskell Setup configure $(CONFIGURE_OPTS)
	env $(BUILD_ENV) runhaskell Setup build
else
	for hsmodule in $(HS_MODULES) ; do \
		( cd $(WRKSRC)/$$hsmodule ; \
		  env $(CONFIGURE_ENV) runhaskell Setup configure $(CONFIGURE_OPTS) ; \
		  env $(BUILD_ENV) runhaskell Setup build ; \
		  env $(BUILD_ENV) runhaskell Setup copy --destdir=$(SYSROOT) ; \
		  env $(BUILD_ENV) runhaskell Setup register --gen-pkg-config=$(SYSROOT)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/$$hsmodule.conf ; \
		  env $(BUILD_ENV) ghc-pkg recache --package-db=$(SYSROOT)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d ; \
		) ; \
		$(MAKE) mangle-package-cache ; \
	done
endif

install:
	mkdir -p $(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d
ifeq ($(HS_MODULES),)
	env $(FAKEINST_ENV) runhaskell Setup copy --destdir=$(DESTDIR)
	env $(FAKEINST_ENV) runhaskell Setup register --gen-pkg-config=$(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/$(PKG)-$(VER).conf
else
	for hsmodule in $(HS_MODULES) ; do \
		( cd $(WRKSRC)/$$hsmodule ; \
		  env $(FAKEINST_ENV) runhaskell Setup copy --destdir=$(DESTDIR) ; \
		  env $(FAKEINST_ENV) runhaskell Setup register --gen-pkg-config=$(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/$$hsmodule.conf ; \
		) ; \
		if [ ! -f $(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d/$$hsmodule.conf ]; then \
			continue ; \
		fi ; \
		( cd $(DESTDIR)$(LIBDIR)/ghc-$(GHC_VER)/package.conf.d ; \
		  sed -i -e '/^haddock-interfaces:/d' $$hsmodule.conf ; \
		  sed -i -e '/^haddock-html:/d' $$hsmodule.conf ; \
		  sed -i -e 's|$(LIBDIR):$(X11_LIBDIR)|$(LIBDIR) $(X11_LIBDIR)|g' $$hsmodule.conf ; \
		  sed -i -e 's|$(INCLUDEDIR):$(X11_INCLUDEDIR)|$(INCLUDEDIR) $(X11_INCLUDEDIR)|g' $$hsmodule.conf ; \
		) ; \
	done
endif
	$(MAKE) unmangle-package-cache
