#!/bin/sh
# {{{ Apache License version 2.0
#
# Copyright 2014 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# Replace all #!/bin/bash instances in the kernel source with #!/bin/sh

[ -d $1 ] || exit 1

# this should take care of most of the bash attempts
find $1 -type f | xargs grep '#!/bin/bash' | cut -d ':' -f 1 | while read line ; do
    sed -i -e 's|#!/bin/bash|#!/bin/sh|g' $line
done

# the doc tree is tricky, make sure it doesn't try to use bash
if [ -d $1/Documentation ]; then
    find $1/Documentation -type f | xargs grep '/bin/bash' | cut -d ':' -f 1 | while read line ; do
        sed -i -e 's|/bin/bash|/bin/sh|g' $line
    done
fi

# dash does not require "SIG" in signal names
# dash also does not recognize 'ERR' as a signal name
if [ -f $1/scripts/link-vmlinux.sh ]; then
    sed -i -e 's|SIG||g' $1/scripts/link-vmlinux.sh
    sed -i -e 's|ERR||g' $1/scripts/link-vmlinux.sh
fi
