/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2018 David Cantrell <david.l.cantrell@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#ifndef _PLIST_H
#define _PLIST_H

#include <sys/queue.h>

/*
 * This describes an entry from a plist file (or attrs file in a .dist).
 * We are not using struct finfo because plist does not represent
 * everything that finfo needs.  The plist file just provides the name,
 * owner, group, and mode.  Everything else is gathered up by the stat(2)
 * used to populate a finfo.
 */
typedef struct _plist_entry_t {
    char *filename;
    char *owner;
    char *group;
    int64_t uid;
    int64_t gid;
    mode_t mode;
    bool is_symlink;
    bool has_checksum;
    TAILQ_ENTRY(_plist_entry_t) items;
} plist_entry_t;

/*
 * Linked list defining an explanded plist file for a package.
 */
typedef TAILQ_HEAD(plist_s, _plist_entry_t) plist_t;

#endif /* _PLIST_H */
