/* Initialize the installed packages database. */

/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2014-2018 David Cantrell <david.l.cantrell@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include "dist-tools.h"
#include "db.h"

/* Initialize the installed packages database if we need to. */
int initdb(char *prepend, const char *dbpath, const char *dbfile) {
    char *err = NULL;
    char *tmp = NULL;
    sqlite3 *db = NULL;

    assert(dbpath != NULL);
    assert(dbfile != NULL);

    /* create the installed package database directory if we need to */
    tmp = joinpath(prepend, dbpath, NULL);

    if (mkdirp(tmp, _DIR_MODE)) {
        fprintf(stderr, "%s (%d): %s\n", __func__, __LINE__, strerror(errno));
        free(tmp);
        return -1;
    }

    /* path to the database file */
    free(tmp);
    tmp = joinpath(prepend, dbfile, NULL);

    /* open database */
    if (sqlite3_open(tmp, &db) != SQLITE_OK) {
        FAIL_SQLITE(db, NULL, -1);
    }

    if (sqlite3_exec(db, "PRAGMA synchronous = OFF", NULL, NULL, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    if (sqlite3_exec(db, "PRAGMA journal_mode = MEMORY", NULL, NULL, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    /* initialize database */
    if (sqlite3_exec(db, DB_PACKAGES_SCHEMA, NULL, 0, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    if (sqlite3_exec(db, DB_CONTENTS_SCHEMA, NULL, 0, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    /* we are finished with the databases here */
    sqlite3_close(db);

    return 0;
}
