/*
 * Makes sure the file is a valid ELF file and is a shared object or an
 * executable.
 */

/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2003-2018 David Cantrell <david.l.cantrell@gmail.com>
 *                     David Shea <david@reallylongword.org>
 *                     Chris Lumens <chris@bangmoney.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <libelf.h>
#include "dist-tools.h"

bool is_elf(const char *filename) {
    int fd;
    bool ret = false;
    Elf *elf = NULL;

    if (filename == NULL) {
        return false;
    }

    elf_version(EV_CURRENT);

    if ((fd = open(filename, O_RDONLY)) == -1) {
        fprintf(stderr, "%s (%d): %s: %s\n", __func__, __LINE__, strerror(errno), filename);
        return false;
    }

    elf = elf_begin(fd, ELF_C_READ_MMAP, NULL);

    if (elf_kind(elf) == ELF_K_ELF) {
        ret = true;
    }

    if (elf_end(elf) != 0) {
        fprintf(stderr, "%s (%d): %s\n", __func__, __LINE__, strerror(errno));
        return false;
    }

    if (close(fd) == -1) {
        fprintf(stderr, "%s (%d): %s\n", __func__, __LINE__, strerror(errno));
        return false;
    }

    return ret;
}
