/*
 * Perform a 'mkdir -p' operation on a given path.
 */

/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2018 David Cantrell <david.l.cantrell@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/stat.h>
#include <sys/types.h>

int mkdirp(char *path, mode_t mode) {
    int r = 0;
    char *p = NULL;
    char *start = NULL;
    struct stat sb;

    assert(path != NULL);
    assert(*path == '/');

    start = p = strdup(path);
    p++;

    /* handle all the parent directories */
    while (*p != '\0') {
        if (*p == '/') {
            *p = '\0';
            r = stat(start, &sb);

            if (r == 0 && S_ISDIR(sb.st_mode)) {
                *p = '/';
                p++;
                continue;
            } else if (mkdir(start, mode) == -1) {
                return -1;
            }

            *p = '/';
        }

        p++;
    }

    /* final directory */
    if ((stat(start, &sb) != 0) && (mkdir(start, mode) == -1)) {
        return -1;
    }

    free(start);

    return 0;
}
