/* Parsing support functions. */

/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2012-2018 David Cantrell <david.l.cantrell@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#include <stdint.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <limits.h>
#include "dist-tools.h"

/*
 * Split plist owner and group fields in to the name and id.
 * Input is a string of the form:  name(ID)
 * where name is a text string and ID is the user or group ID number.
 * The parentheses are for human readability, so we parse those out.
 */
int split_name_id(char *field, char **name, uint64_t *id) {
    char *tmp = field;

    assert(field != NULL);

    if ((*name = calloc(1, sizeof(char *))) == NULL) {
        fprintf(stderr, "%s (%d): %s\n", __func__, __LINE__, strerror(errno));
        return -1;
    }

    for (tmp = field; *tmp != '('; tmp++);
    *tmp = '\0';
    *name = strdup(field);
    field += strlen(field) + 1;
    tmp = field;

    for (tmp = field; *tmp != ')'; tmp++);
    *tmp = '\0';
    errno = 0;
    *id = strtoull(field, NULL, 10);
    if ((errno == ERANGE && *id == ULLONG_MAX) || (errno == EINVAL && *id == 0)) {
        return -1;
    }

    return 0;
}
