/* Remove package entry from the database. */

/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2014-2018 David Cantrell <david.l.cantrell@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "dist-tools.h"
#include "db.h"

extern char *db_installed;

/*
 * Given a struct dist, remove its entry from the package database
 * and all files that belong to its package ID.
 */
int rmentry(const struct dist *package) {
    char *err = NULL;
    char *cmd = NULL;
    sqlite3 *db = NULL;

    assert(package != NULL);
    assert(package->id >= 0);

    /* open the installed package database */
    if (sqlite3_open(db_installed, &db) != SQLITE_OK) {
        FAIL_SQLITE(db, NULL, -1);
    }

    if (sqlite3_exec(db, "PRAGMA synchronous = OFF", NULL, NULL, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    if (sqlite3_exec(db, "PRAGMA journal_mode = MEMORY", NULL, NULL, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    /* remove the entry from the packages table */
    xasprintf(&cmd, "DELETE FROM packages WHERE id='%d'", package->id);

    if (sqlite3_exec(db, cmd, NULL, 0, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    free(cmd);
    cmd = NULL;

    /* remove the entries from the contents table */
    xasprintf(&cmd, "DELETE FROM contents WHERE packageid='%d'", package->id);

    if (sqlite3_exec(db, cmd, NULL, 0, &err) != SQLITE_OK) {
        FAIL_SQLITE(db, err, -1);
    }

    free(cmd);
    cmd = NULL;

    /* clean up */
    sqlite3_close(db);

    return 0;
}
