/*
 * Append second string to first string, resize memory as necessary.  The
 * first argument must be a dynamically allocated string or NULL.  If NULL,
 * this function will allocate memory for the operation.  The caller must
 * free the result in any case.
 *
 * Second string must not be NULL.
 *
 * Also a helper function to append an int to a string by first converting
 * the int to a string and then strcat'ing that.
 */

/* {{{ Apache License version 2.0
 */
/*
 * Copyright 2018 David Cantrell <david.l.cantrell@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* }}} */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "dist-tools.h"

char *strappend(char *first, char *second) {
    assert(second != NULL);

    if (first == NULL) {
        return strdup(second);
    }

    first = realloc(first, strlen(first) + strlen(second) + 1);
    return strcat(first, second);
}

char *strappend_int(char *str, int i) {
    char *r = NULL;
    char *tmp = NULL;

    xasprintf(&tmp, "%i", i);
    str = realloc(str, strlen(str) + strlen(tmp) + 1);
    r = strcat(str, tmp);
    free(tmp);
    return r;
}
