#!/bin/sh
# Verify distadd checks ELF shared dependencies.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG="test-1.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the test package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# distadd deps check (missing deps)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc
touch ${OUTPUT}/r/etc/ld.so.conf
/sbin/ldconfig -r ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd ELF dependency checking (missing deps)..."
    ${LEAKCHECK_CMD} ${DISTADD} -r ${OUTPUT}/r ${OUTPUT}/${PKG}
else
    echo -n "checking distadd ELF dependency checking (missing deps)..."
    out="$(${DISTADD} -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep -E "^\*\*\*\ lib.*\ not\ found")"
    if [ ${r} -eq 1 -a ! -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# distadd deps check skip
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd ELF dependency checking (skip depcheck)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
else
    echo -n "checking distadd ELF dependency checking (skip depcheck)..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep -E "^\*\*\*\ lib.*\ not\ found")"
    if [ ${r} -eq 0 -a -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected zero failures
    [ ${FAILS} -gt 0 ] && exit 1 || exit 0
fi
