#!/bin/sh
# Verify 'distadd -g' functionality works.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
DISTCHK=$(readlink -f ../distchk)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${DISTCHK} ] && ${MAKE} -C ../ distchk

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
( cd ${TREE}/usr/sbin ; ln -s distmk mkdist )

# create plist
u="$(id -un)($(id -u))"
g="$(id -gn)($(id -g))"
echo "usr $u $g 0755" > ${META}/plist
echo "usr/sbin $u $g 0755" >> ${META}/plist
echo "usr/sbin/distmk $u $g 0755" >> ${META}/plist
echo "usr/sbin/mkdist $u $g symlink(distmk)" >> ${META}/plist

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
PKG="test-1.0-1.$(uname -m).dist"

# verify functionality
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -g PKG' generates attributes..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${DISTCHK} -r ${OUTPUT}/r test
    rm -rf ${OUTPUT}/r
    mkdir -p ${OUTPUT}/r
    ${LEAKCHECK_CMD} ${DISTADD} -d -g -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${DISTCHK} -r ${OUTPUT}/r test
else
    echo -n "checking 'distadd -g PKG' generates attributes..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    ${DISTCHK} -r ${OUTPUT}/r test >/dev/null 2>&1
    r1=$?
    rm -rf ${OUTPUT}/r
    mkdir -p ${OUTPUT}/r
    ${DISTADD} -d -g -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    ${DISTCHK} -r ${OUTPUT}/r test >/dev/null 2>&1
    r2=$?
    if [ ${r1} -ne ${r2} ]; then
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    else
        echo "passed."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
