#!/bin/sh
# Verify distadd can handle hard links.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp ${DISTMK} ${TREE}/usr/sbin/distmk
( cd ${TREE}/usr/sbin ; ln distmk distmk2 )

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make a variant package with different file symlinks
( cd ${TREE}/usr/sbin ; rm -f distmk2 )
( cd ${TREE}/usr/sbin ; ln distmk distmk3 )
( cd ${TREE}/usr/sbin ; ln distmk3 distmk2 )
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
usr/sbin/distmk3 root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# install package with hard links
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd' on package with hard links..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd' on package with hard links..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1 | tail -n 1)"
    r=$?
    if [ $r -eq 0 -a "$out" = "Installed package $(basename ${PKG1} .dist) successfully" -a -x ${OUTPUT}/r/usr/sbin/distmk -a -x ${OUTPUT}/r/usr/sbin/distmk2 -a $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk) -eq 2 -a $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk2) -eq 2 -a "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk)" = "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk2)" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade a package with hard links
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u' on a package with hard links..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd -u' on a package with hard links..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1 | tail -n 1)"
    r=$?
    if [ $r -eq 0 -a "$out" = "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG2} .dist) successfully" -a -x ${OUTPUT}/r/usr/sbin/distmk -a -x ${OUTPUT}/r/usr/sbin/distmk2 -a $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk) -eq 3 -a $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk2) -eq 3 -a $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk3) -eq 3 -a "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk)" = "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk2)" -a "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk)" = "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk3)" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
