#!/bin/sh
# Verify distadd handles multiple packages specified on the command line.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
DISTINFO=$(readlink -f ../distinfo)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="foo-1.0-1.$(uname -m).dist"
PKG2="bar-2.0-1.$(uname -m).dist"
PKG3="qux-3.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${DISTINFO} ] && ${MAKE} -C ../ distinfo

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "foo" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "foo" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

rm -f ${TREE}/usr/sbin/distmk
cp -p ${DISTADD} ${TREE}/usr/sbin/distadd

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distadd root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "bar" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "bar" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

rm -f ${TREE}/usr/sbin/distadd
cp -p ${DISTINFO} ${TREE}/usr/sbin/distinfo

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distinfo root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "qux" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "qux" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

#####################
# multiple packages #
#####################

# install multiple packages specified on the command line
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd PKG1 PKG2 PKG3' works..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd PKG1 PKG2 PKG3' works..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    out1="$(echo "${out}" | grep "Installed package foo")"
    out2="$(echo "${out}" | grep "Installed package bar")"
    out3="$(echo "${out}" | grep "Installed package qux")"
    if [ ${r} -eq 0 -a "${out1}" = "Installed package $(basename ${PKG1} .dist) successfully" -a "${out2}" = "Installed package $(basename ${PKG2} .dist) successfully" -a "${out3}" = "Installed package $(basename ${PKG3} .dist) successfully" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade multiple packages specified on the command line should fail
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u PKG1 PKG2 PKG3' skips if packages are installed..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u PKG1 PKG2 PKG3' skips if packages are installed..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    out1="$(echo "${out}" | grep "Package foo")"
    out2="$(echo "${out}" | grep "Package bar")"
    out3="$(echo "${out}" | grep "Package qux")"
    if [ ${r} -eq 0 -a "${out1}" = "Package foo-1.0-1 already installed, skipping." -a "${out2}" = "Package bar-2.0-1 already installed, skipping." -a "${out3}" = "Package qux-3.0-1 already installed, skipping." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade force multiple packages should work
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -f PKG1 PKG2 PKG3' succeeds..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -f -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u -f PKG1 PKG2 PKG3' succeeds..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -f -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    out1="$(echo "${out}" | grep "Force upgraded package foo")"
    out2="$(echo "${out}" | grep "Force upgraded package bar")"
    out3="$(echo "${out}" | grep "Force upgraded package qux")"
    if [ ${r} -eq 0 -a "${out1}" = "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG1} .dist) successfully" -a "${out2}" = "Force upgraded package $(basename ${PKG2} .dist) to $(basename ${PKG2} .dist) successfully" -a "${out3}" = "Force upgraded package $(basename ${PKG3} .dist) to $(basename ${PKG3} .dist) successfully" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
