#!/bin/sh
# Verify distadd can install and upgrade multiversion packages.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-2.0-2.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make a variant package to upgrade to
mv ${TREE}/usr/sbin/distmk ${TREE}/usr/sbin/distmk2
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make another variant to upgrade to
cat << EOF > ${TREE}/usr/sbin/mkdist
#!/bin/sh
/usr/sbin/distmk2 $*
EOF
chmod 0755 ${TREE}/usr/sbin/mkdist

echo "usr/sbin/mkdist root(0) root(0) 0755" >> ${META}/plist

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 2 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 2 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# multi version package install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -m'..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking 'distadd -m'..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
    p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|1.0|1")"
    p3="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "2|test|2.0|1")"
    if [ ${r} -eq 0 -a ${p1} -eq 2 -a ! -z "${p2}" -a ! -z "${p3}" -a -x ${OUTPUT}/r/usr/sbin/distmk -a -x ${OUTPUT}/r/usr/sbin/distmk2 ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade with -P, package already installed
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -P' reports package already installed..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -P ${spec} -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd -u -P' reports package already installed..."
    spec="$(basename ${PKG1} .dist)"
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -P ${spec} -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    if [ ${r} -eq 0 -a "${out}" = "Package test-1.0-1 already installed, skipping." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade with -P and force, package installs
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -f -P' installs the package..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -f -P ${spec} -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd -u -f -P' installs the package..."
    spec="$(basename ${PKG1} .dist)"
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -f -P ${spec} -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "Force upgraded package ${spec} to ${spec} successfully" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade with -P -f -m, package installs
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -f -m -P' installs the package..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -f -m -P ${spec} -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd -u -f -m -P' installs the package..."
    spec="$(basename ${PKG1} .dist)"
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -f -m -P ${spec} -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "Force upgraded package ${spec} to ${spec} successfully" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# make sure the -P is required with -u -m
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -m' requires '-P'..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -m -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u -m' requires '-P'..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -m -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    if [ "${out}" = "The -P or -v option is required when specifying -u and -m." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# make sure the -P and -v options cannot be used together
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -v' and 'distadd -P' cannot be used together..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -m -v 2.0 -P test-2.0 -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -v' and 'distadd -P' cannot be used together..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -m -v 2.0 -P test-2.0 -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    if [ "${out}" = "The -P and -v options cannot be used together." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# multi version package upgrade with package spec
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -m -P'..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -m -P test-2.0 -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u -m -P'..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -m -P test-2.0 -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "Upgraded package test-2.0-1.$(uname -m) to test-2.0-2.$(uname -m) successfully" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# multi version package upgrade with package version
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -m -v'..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -m -v "2.0" -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u -m -v'..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -m -v "2.0" -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
    p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|1.0|1")"
    p3="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "2|test|2.0|2")"
    if [ ${r} -eq 0 -a -x ${OUTPUT}/r/usr/sbin/distmk -a -x ${OUTPUT}/r/usr/sbin/distmk2 -a -x ${OUTPUT}/r/usr/sbin/mkdist -a ${p1} -eq 2 -a ! -z "${p2}" -a ! -z "${p3}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# multi version package upgrade with package spec
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -m -P'..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -m -P "test-2.0" -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u -m -P'..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -m -P "test-2.0" -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
    p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|1.0|1")"
    p3="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "2|test|2.0|2")"
    if [ ${r} -eq 0 -a -x ${OUTPUT}/r/usr/sbin/distmk -a -x ${OUTPUT}/r/usr/sbin/distmk2 -a -x ${OUTPUT}/r/usr/sbin/mkdist -a ${p1} -eq 2 -a ! -z "${p2}" -a ! -z "${p3}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
