#!/bin/sh
# Verify distadd handles package scripts correctly.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
DISTRM=$(readlink -f ../distrm)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${DISTRM} ] && ${MAKE} -C ../ distrm

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

cat << EOF > ${META}/preremove
#!/bin/sh
if [ "\${ACTION}" = "remove" ]; then
    echo "*** PREREMOVE RUNS ON REMOVE ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** PREREMOVE RUNS ON UPGRADE ***"
fi
echo "VER=\${VER}"
echo "UPGRADEVER=\${UPGRADEVER}"
EOF

chmod 0755 ${META}/preremove
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/preremove

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

#############
# preremove #
#############

# preremove execution on remove
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preremove script execution on remove..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking distadd preremove script execution on remove..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    msg="$(echo "${out}" | grep '*** PREREMOVE RUNS ON REMOVE ***')"
    ver="$(echo "${out}" | grep ^VER= | cut -d '=' -f 2)"
    upgradever="$(echo "${out}" | grep ^UPGRADEVER= | cut -d '=' -f 2)"
    if [ ${r} -eq 0 -a "${msg}" = "*** PREREMOVE RUNS ON REMOVE ***" -a "${ver}" = "1.0" -a -z "${upgradever}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preremove execution on upgrade
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preremove script execution on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd preremove script execution on upgrade..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    msg="$(echo "${out}" | grep '*** PREREMOVE RUNS ON UPGRADE ***')"
    if [ ${r} -eq 0 -a "${msg}" = "*** PREREMOVE RUNS ON UPGRADE ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preremove execution on upgrade, VER != UPGRADEVER
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preremove script execution on upgrade (VER != UPGRADEVER)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd preremove script execution on upgrade (VER != UPGRADEVER)..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    msg="$(echo "${out}" | grep '*** PREREMOVE RUNS ON UPGRADE ***')"
    ver="$(echo "${out}" | grep ^VER= | cut -d '=' -f 2)"
    upgradever="$(echo "${out}" | grep ^UPGRADEVER= | cut -d '=' -f 2)"
    if [ ${r} -eq 0 -a "${msg}" = "*** PREREMOVE RUNS ON UPGRADE ***" -a "${ver}" = "1.0" -a "${upgradever}" = "2.0" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
