#!/bin/sh
# Verify distadd handles package scripts correctly.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"
PKG4="test-4.0-1.$(uname -m).dist"
PKG5="test-5.0-1.$(uname -m).dist"
PKG6="test-6.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

cat << EOF > ${META}/preinstall
#!/bin/sh
if [ "\${ACTION}" = "install" ]; then
    echo "*** PREINSTALL RUNS ON INSTALL ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** PREINSTALL RUNS ON UPGRADE ***"
fi
EOF

cat << EOF > ${META}/postinstall
#!/bin/sh
if [ "\${ACTION}" = "install" ]; then
    echo "*** POSTINSTALL RUNS ON INSTALL ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** POSTINSTALL RUNS ON UPGRADE ***"
fi
EOF

cat << EOF > ${META}/preremove
#!/bin/sh
if [ "\${ACTION}" = "remove" ]; then
    echo "*** PREREMOVE RUNS ON REMOVE ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** PREREMOVE RUNS ON UPGRADE ***"
fi
EOF

chmod 0755 ${META}/preinstall
chmod 0755 ${META}/postinstall
chmod 0755 ${META}/preremove

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}

else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

rm -f ${META}/preinstall
rm -f ${META}/postinstall
rm -f ${META}/preremove

cat << EOF > ${META}/preinstall
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/preinstall
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/preinstall

cat << EOF > ${META}/postinstall
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/postinstall
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "4.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "4.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/postinstall

cat << EOF > ${META}/preremove
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/preremove
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "5.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "6.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "5.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
    ${DISTMK} -t ${TREE} -p "test" -v "6.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/preremove

##############
# preinstall #
##############

# preinstall execution on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preinstall script execution on install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking distadd preinstall script execution on install..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** PREINSTALL RUNS ON INSTALL ***')"
    if [ ${r} -eq 0 -a "${out}" = "*** PREINSTALL RUNS ON INSTALL ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preinstall execution on upgrade
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preinstall script execution on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd preinstall script execution on upgrade..."
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** PREINSTALL RUNS ON UPGRADE ***')"
    if [ ${r} -eq 0 -a "${out}" = "*** PREINSTALL RUNS ON UPGRADE ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preinstall skip on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preinstall script skip on install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -S -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking distadd preinstall script skip on install..."
    out="$(${DISTADD} -d -S -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** PREINSTALL RUNS ON INSTALL ***')"
    if [ ${r} -eq 0 -a "${out}" = "" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preinstall skip on upgrade
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preinstall script skip on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -S -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd preinstall script skip on upgrade..."
    out="$(${DISTADD} -d -f -S -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** PREINSTALL RUNS ON UPGRADE ***')"
    if [ ${r} -eq 0 -a -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preinstall fails, but continue install
rm -rf ${OUTPUT}/r2
mkdir -p ${OUTPUT}/r2

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -F' on preinstall script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -F -r ${OUTPUT}/r2 ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -F' on preinstall script failure..."
    out="$(${DISTADD} -d -F -r ${OUTPUT}/r2 ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "preinstall failed for test, continuing anyway")"
    if [ ${r} -eq 0 -a ! -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

###############
# postinstall #
###############

# postinstall execution on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd postinstall script execution on install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking distadd postinstall script execution on install..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** POSTINSTALL RUNS ON INSTALL ***')"
    if [ ${r} -eq 0 -a "${out}" = "*** POSTINSTALL RUNS ON INSTALL ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# postinstall execution on upgrade
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd postinstall script execution on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd postinstall script execution on upgrade..."
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** POSTINSTALL RUNS ON UPGRADE ***')"
    if [ ${r} -eq 0 -a "${out}" = "*** POSTINSTALL RUNS ON UPGRADE ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# postinstall skip on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd postinstall script skip on install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -S -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking distadd postinstall script skip on install..."
    out="$(${DISTADD} -d -S -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** POSTINSTALL RUNS ON INSTALL ***')"
    if [ ${r} -eq 0 -a "${out}" = "" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# postinstall skip on upgrade
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd postinstall script skip on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -S -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd postinstall script skip on upgrade..."
    out="$(${DISTADD} -d -f -S -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** POSTINSTALL RUNS ON UPGRADE ***')"
    if [ ${r} -eq 0 -a "${out}" = "" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# postinstall fails, but continue install
rm -rf ${OUTPUT}/r2
mkdir -p ${OUTPUT}/r2

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -F' on postinstall script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -F -r ${OUTPUT}/r2 ${OUTPUT}/${PKG4}
else
    echo -n "checking 'distadd -F' on postinstall script failure..."
    out="$(${DISTADD} -d -F -r ${OUTPUT}/r2 ${OUTPUT}/${PKG4} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "postinstall failed for test, continuing anyway")"
    if [ ${r} -eq 0 -a ! -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

#############
# preremove #
#############

# preremove execution on upgrade
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preremove script execution on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd preremove script execution on upgrade..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** PREREMOVE RUNS ON UPGRADE ***')"
    if [ ${r} -eq 0 -a "${out}" = "*** PREREMOVE RUNS ON UPGRADE ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preremove skip on upgrade
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distadd preremove script skip on upgrade..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -S -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking distadd preremove script skip on upgrade..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -S -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep '*** PREREMOVE RUNS ON REMOVE ***')"
    if [ ${r} -eq 0 -a "${out}" = "" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# preremove fails, but continue install
rm -rf ${OUTPUT}/r2
mkdir -p ${OUTPUT}/r2

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -F' on preremove script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r2 ${OUTPUT}/${PKG5}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -F -u -r ${OUTPUT}/r2 ${OUTPUT}/${PKG6}
else
    echo -n "checking 'distadd -F' on preremove script failure..."
    ${DISTADD} -d -r ${OUTPUT}/r2 ${OUTPUT}/${PKG5} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -F -u -r ${OUTPUT}/r2 ${OUTPUT}/${PKG6} 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "preremove failed for test, continuing anyway")"
    if [ ${r} -eq 0 -a ! -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
