#!/bin/sh
# Verify distadd can handle symlinks.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
( cd ${TREE}/usr/sbin ; ln -s distmk distmk2 )

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) symlink(distmk)
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make a variant package with different file symlinks
( cd ${TREE}/usr/sbin ; ln -sf distmk distmk3 )
( cd ${TREE}/usr/sbin ; ln -sf distmk3 distmk2 )
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) symlink(distmk3)
usr/sbin/distmk3 root(0) root(0) symlink(distmk)
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make a variant package with directory symlinks
rm -f ${TREE}/usr/sbin/distmk2
rm -f ${TREE}/usr/sbin/distmk3
( cd ${TREE}/usr ; ln -sf sbin sbin2 )
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin2 root(0) root(0) symlink(sbin)
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# install package with symlinks
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd' on package with file symlinks..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd' on package with file symlinks..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1 | tail -n 1)"
    r=$?
    if [ $r -eq 0 -a "$out" = "Installed package $(basename ${PKG1} .dist) successfully" -a -x ${OUTPUT}/r/usr/sbin/distmk -a -L ${OUTPUT}/r/usr/sbin/distmk2 -a "$(readlink ${OUTPUT}/r/usr/sbin/distmk2)" = "distmk" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade a package with symlinks to a package with symlinks
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u' on a package with file symlinks..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking 'distadd -u' on a package with file symlinks..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1 | tail -n 1)"
    r=$?
    if [ $r -eq 0 -a "$out" = "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG2} .dist) successfully" -a -x ${OUTPUT}/r/usr/sbin/distmk -a -L ${OUTPUT}/r/usr/sbin/distmk2 -a -L ${OUTPUT}/r/usr/sbin/distmk3 -a "$(readlink ${OUTPUT}/r/usr/sbin/distmk2)" = "distmk3" -a "$(readlink ${OUTPUT}/r/usr/sbin/distmk3)" = "distmk" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade a package with symlinks to a package with directory symlinks
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u' on a package with directory symlinks..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u' on a package with directory symlinks..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1 | tail -n 1)"
    r=$?
    if [ $r -eq 0 -a "$out" = "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG3} .dist) successfully" -a -x ${OUTPUT}/r/usr/sbin/distmk -a ! -r ${OUTPUT}/r/usr/sbin/distmk2 -a -L ${OUTPUT}/r/usr/sbin2 -a "$(readlink ${OUTPUT}/r/usr/sbin2)" = "sbin" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
