#!/bin/sh
# Verify distadd can upgrade packages.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make a variant package to upgrade to
mv ${TREE}/usr/sbin/distmk ${TREE}/usr/sbin/distmk2
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# make a variant package to upgrade to (same filename)
echo "different file" > ${TREE}/usr/sbin/distmk2

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# upgrade same package to same package (error)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u' on same NVR..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd -u' on same NVR..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    if [ ${r} -eq 0 -a "${out}" = "Package test-1.0-1 already installed, skipping." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade same package to same package (force)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u -f' on same NVR..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -u -f -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
else
    echo -n "checking 'distadd -u -f' on same NVR..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -u -f -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM packages" | wc -l)
    if [ ${r} -eq 0 -a "${out}" = "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG1} .dist) successfully" -a -f ${OUTPUT}/r/usr/sbin/distmk -a ${p1} -eq 1 ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade to new package
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u' to new NVR..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
else
    echo -n "checking 'distadd -u' to new NVR..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
    p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|2.0|1")"
    if [ ${r} -eq 0 -a "${out}" = "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG2} .dist) successfully" -a -f ${OUTPUT}/r/usr/sbin/distmk2 -a ! -f ${OUTPUT}/r/usr/sbin/distmk -a ${p1} -eq 1 -a ! -z "${p2}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# upgrade to new package (same filename)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd -u' to new NVR (same filename)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
else
    echo -n "checking 'distadd -u' to new NVR (same filename)..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
    p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|3.0|1")"
    if [ ${r} -eq 0 -a "${out}" = "Force upgraded package $(basename ${PKG2} .dist) to $(basename ${PKG3} .dist) successfully" -a -f ${OUTPUT}/r/usr/sbin/distmk2 -a ${p1} -eq 1 -a ! -z "${p2}" -a "$(cat ${OUTPUT}/r/usr/sbin/distmk2)" = "different file" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
