#!/bin/sh
# Verify we can use wildcards with the distadd command.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0
PKG1="foo-1.0-1.$(uname -m).dist"
PKG2="bar-2.0-1.$(uname -m).dist"
PKG3="baz-3.0-1.$(uname -m).dist"
PKG4="qux-4.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd

# meta files
echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# PKG1:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "foo" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "foo" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# PKG2:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/bin
cp -p ${DISTMK} ${TREE}/usr/bin/distmk

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/bin root(0) root(0) 0755
usr/bin/distmk root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "bar" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "bar" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# PKG3:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTADD} ${TREE}/usr/sbin/distadd

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distadd root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "baz" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "baz" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# PKG4:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/bin
cp -p ${DISTADD} ${TREE}/usr/bin/distadd

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/bin root(0) root(0) 0755
usr/bin/distadd root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "qux" -v "4.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "qux" -v "4.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# install multiple packages by * wildcard
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd *.dist' on package files..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/*.dist
else
    echo -n "checking 'distadd *.dist' on package files..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/*.dist 2>&1)"
    r=$?
    count="$(echo "${out}" | wc -l)"
    echo "${out}" | grep -q "Installed package foo-1.0-1" >/dev/null 2>&1
    f1=$?
    echo "${out}" | grep -q "Installed package bar-2.0-1" >/dev/null 2>&1
    f2=$?
    echo "${out}" | grep -q "Installed package baz-3.0-1" >/dev/null 2>&1
    f3=$?
    echo "${out}" | grep -q "Installed package qux-4.0-1" >/dev/null 2>&1
    f4=$?
    if [ ${r} -eq 0 -a ${count} -eq 8 -a ${f1} -eq 0 -a ${f2} -eq 0 -a ${f3} -eq 0 -a ${f4} -eq 0 ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# install multiple packages with {} brace expansion
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distadd {}' expansion on package files..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/{foo,qux}*dist
else
    echo -n "checking 'distadd {}' expansion on package files..."
    out="$(${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/{foo,qux}*dist 2>&1)"
    r=$?
    count="$(echo "${out}" | wc -l)"
    echo "${out}" | grep -q "Installed package foo-1.0-1" >/dev/null 2>&1
    f1=$?
    echo "${out}" | grep -q "Installed package bar-2.0-1" >/dev/null 2>&1
    f2=$?
    echo "${out}" | grep -q "Installed package baz-3.0-1" >/dev/null 2>&1
    f3=$?
    echo "${out}" | grep -q "Installed package qux-4.0-1" >/dev/null 2>&1
    f4=$?
    if [ ${r} -eq 0 -a ${count} -eq 4 -a ${f1} -eq 0 -a ${f2} -eq 1 -a ${f3} -eq 1 -a ${f4} -eq 0 ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
