#!/bin/sh
# Verify distchk works on packages with skipped checksums for files.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
DISTCHK=$(readlink -f ../distchk)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${DISTCHK} ] && ${MAKE} -C ../ distchk

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
cp -p ${DISTMK} ${TREE}/usr/sbin/mkdist

# create plist
USER="$(id -un)"
tUID="$(id -ur)"
GROUP="$(id -gn)"
tGID="$(id -gr)"

echo "usr ${USER}(${tUID}) ${GROUP}(${tGID}) 0755" > ${META}/plist
echo "usr/sbin ${USER}(${tUID}) ${GROUP}(${tGID}) 0755" >> ${META}/plist
echo "usr/sbin/distmk ${USER}(${tUID}) ${GROUP}(${tGID}) 0755 nochecksum" >> ${META}/plist
echo "usr/sbin/mkdist ${USER}(${tUID}) ${GROUP}(${tGID}) 0755" >> ${META}/plist

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

PKG="test-1.0-1.$(uname -m).dist"

# check installed package with nochecksum used
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distchk PKG' on a nochecksum package..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${DISTCHK} -r ${OUTPUT}/r test
else
    echo -n "checking 'distchk PKG' on a nochecksum package..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${DISTCHK} -r ${OUTPUT}/r test 2>&1 | tail -n 1)"
    r=$?
    if [ ${r} -eq 0 -a "${out}" = "OK" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    [ ${FAILS} -gt 0 ] && exit 1 || exit 0
fi
