#!/bin/sh
# Verify the --deps option works on distinfo.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTINFO=$(readlink -f ../distinfo)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTINFO} ] && ${MAKE} -C ../ distinfo

# create a tree to package up
mkdir -p ${TREE}/usr/bin
cp -p ${DISTMK} ${TREE}/usr/bin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/bin root(0) root(0) 0755
usr/bin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

PKG="test-1.0-1.$(uname -m).dist"

# list the ELF shared libraries required by this package
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distinfo --deps PKG' on a package file..."
    ${LEAKCHECK_CMD} ${DISTINFO} --deps ${OUTPUT}/${PKG}
else
    echo -n "checking 'distinfo --deps PKG' on a package file..."
    out="$(${DISTINFO} --deps ${OUTPUT}/${PKG} 2>&1)"
    r=$?
    numdeps=$(echo "${out}" | wc -l)
    echo "${out}" | grep -q "^libarchive" >/dev/null 2>&1
    haslibarchive=$?
    echo "${out}" | grep -q "^liblzma" >/dev/null 2>&1
    hasliblzma=$?
    echo "${out}" | grep -q "^libelf" >/dev/null 2>&1
    haslibelf=$?
    echo "${out}" | grep -q "^libsqlite" >/dev/null 2>&1
    haslibsqlite=$?
    echo "${out}" | grep -q "^libssl" >/dev/null 2>&1
    haslibssl=$?
    echo "${out}" | grep -q "^libcrypto" >/dev/null 2>&1
    haslibcrypto=$?
    echo "${out}" | grep -q "^libc.so.6" >/dev/null 2>&1
    haslibc=$?
    if [ ${r} -eq 0 -a ${numdeps} -eq 7 -a ${haslibarchive} -eq 0 -a ${hasliblzma} -eq 0 -a ${haslibelf} -eq 0 -a ${haslibsqlite} -eq 0 -a ${haslibssl} -eq 0 -a ${haslibcrypto} -eq 0 -a ${haslibc} -eq 0 ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# ensure --deps only works for package files
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distinfo --deps PKG' on an installed package..."
    ${LEAKCHECK_CMD} ${DISTINFO} --deps glibc
else
    echo -n "checking 'distinfo --deps PKG' on an installed package..."
    out="$(${DISTINFO} --deps glibc 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "The --provs and --deps options are only valid on package files." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
