#!/bin/sh
# Verify the --provs option works on distinfo.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTINFO=$(readlink -f ../distinfo)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTINFO} ] && ${MAKE} -C ../ distinfo

# create a tree to package up
mkdir -p ${TREE}/usr/lib
cp /usr/lib/libc.so.6 ${TREE}/usr/lib/libc.so.6
chmod 0755 ${TREE}/usr/lib/libc.so.6

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/lib root(0) root(0) 0755
usr/lib/libc.so.6 root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

PKG="test-1.0-1.$(uname -m).dist"

# list the ELF shared libraries provided by this package
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distinfo --provs PKG' on a package file..."
    ${LEAKCHECK_CMD} ${DISTINFO} --provs ${OUTPUT}/${PKG}
else
    echo -n "checking 'distinfo --provs PKG' on a package file..."
    out="$(${DISTINFO} --provs ${OUTPUT}/${PKG} 2>&1)"
    r=$?
    if [ ${r} -eq 0 -a "${out}" = "libc.so.6" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# ensure --provs only works for package files
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distinfo --provs PKG' on an installed package..."
    ${LEAKCHECK_CMD} ${DISTINFO} --provs glibc
else
    echo -n "checking 'distinfo --provs PKG' on an installed package..."
    out="$(${DISTINFO} --provs glibc 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "The --provs and --deps options are only valid on package files." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
