#!/bin/sh
# Verify we can use wildcards with the distinfo command.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTINFO=$(readlink -f ../distinfo)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTINFO} ] && ${MAKE} -C ../ distinfo

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the packages
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null
    ${LEAKCHECK_CMD} \ 
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi

# list info about package files
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distinfo *.dist' on package files..."
    ${LEAKCHECK_CMD} ${DISTINFO} ${OUTPUT}/*.dist
else
    echo -n "checking 'distinfo *.dist' on package files..."
    out="$(${DISTINFO} ${OUTPUT}/*.dist 2>&1)"
    r=$?
    count="$(echo "${out}" | wc -l)"
    echo "${out}" | grep -q "test-1.0" >/dev/null 2>&1
    f1=$?
    echo "${out}" | grep -q "test-2.0" >/dev/null 2>&1
    f2=$?
    echo "${out}" | grep -q "test-3.0" >/dev/null 2>&1
    f3=$?
    if [ ${r} -eq 0 -a ${count} -eq 3 -a ${f1} -eq 0 -a ${f2} -eq 0 -a ${f3} -eq 0 ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
