#!/bin/sh
# Make sure all compression types are honored

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

CWD=$(pwd)

# create the package
ARGS="-t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}"
PKG="test-1.0-1.$(uname -m).dist"

# bzip2
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking bzip2 support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z bzip2
else
    echo -n "checking bzip2 support..."
    ${DISTMK} ${ARGS} -z bzip2 >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-bzip2$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# bz2
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking bz2 support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z bz2
else
    echo -n "checking bz2 support..."
    ${DISTMK} ${ARGS} -z bz2 >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-bzip2$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# gzip
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking gzip support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z gzip
else
    echo -n "checking gzip support..."
    ${DISTMK} ${ARGS} -z gzip >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-gzip$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# gz
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking gz support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z gz
else
    echo -n "checking gz support..."
    ${DISTMK} ${ARGS} -z gz >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-gzip$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# compress
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking compress support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z compress
else
    echo -n "checking compress support..."
    ${DISTMK} ${ARGS} -z compress >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-compress$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# Z
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking Z support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z Z
else
    echo -n "checking Z support..."
    ${DISTMK} ${ARGS} -z Z >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-compress$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# xz
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking xz support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z xz
else
    echo -n "checking xz support..."
    ${DISTMK} ${ARGS} -z xz >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-xz$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# lzma
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking lzma support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z lzma
else
    echo -n "checking lzma support..."
    ${DISTMK} ${ARGS} -z lzma >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-lzma$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

# none
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking none (disable payload compression) support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -z none
else
    echo -n "checking none (disable payload compression) support..."
    ${DISTMK} ${ARGS} -z none >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    file -b --mime-type reloc.payload 2>&1 | grep -q -E "^application/x-tar$" >/dev/null 2>&1
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected zero failures
    [ ${FAILS} -gt 0 ] && exit 1 || exit 0
fi
