#!/bin/sh
# Make sure 'nochecksum' is honored in the plist file.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
cp -p ${DISTMK} ${TREE}/usr/sbin/mkdist

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755 nochecksum
usr/sbin/mkdist root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

CWD=$(pwd)

# create the package
ARGS="-t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}"
PKG="test-1.0-1.$(uname -m).dist"

# nochecksum
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'nochecksum' support..."
    ${LEAKCHECK_CMD} ${DISTMK} ${ARGS} -s sha256
else
    echo -n "checking 'nochecksum' support..."
    ${DISTMK} ${ARGS} -s sha256 >/dev/null 2>&1
    cd ${OUTPUT}
    ar x ${PKG} >/dev/null 2>&1
    sum1="$(sqlite bom.db "SELECT digest FROM contents WHERE path='usr/sbin/distmk'")"
    sum2="$(sqlite bom.db "SELECT digest FROM contents WHERE path='usr/sbin/mkdist'")"
    computed="$(sha256sum ${TREE}/usr/sbin/mkdist | cut -d ' ' -f 1)"
    if [ "${sum1}" = "*" -a "${sum2}" = "${computed}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi
cd ${CWD}
rm -rf ${OUTPUT}/*

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected zero failures
    [ ${FAILS} -gt 0 ] && exit 1 || exit 0
fi
