#!/bin/sh
# Verify the required options on distmk are required.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
FAILS=0
TREE="$(mktemp -d)"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distmk -t'..."
    ${LEAKCHECK_CMD} ${DISTMK}
else
    echo -n "checking 'distmk -t'..."
    out="$(${DISTMK} 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "*** ERROR: Missing package root tree (-t)." ]; then
        FAILS=$((${FAILS} + ${r}))
        echo "passed."
    else
        echo "FAILED."
    fi
fi

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distmk -p'..."
    ${LEAKCHECK_CMD} ${DISTMK} -t ${TREE}
else
    echo -n "checking 'distmk -p'..."
    out="$(${DISTMK} -t ${TREE} 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "*** ERROR: Missing package name (-p)." ]; then
        FAILS=$((${FAILS} + ${r}))
        echo "passed."
    else
        echo "FAILED."
    fi
fi

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distmk -v'..."
    ${LEAKCHECK_CMD} ${DISTMK} -t ${TREE} -p "test"
else
    echo -n "checking 'distmk -v'..."
    out="$(${DISTMK} -t ${TREE} -p "test" 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "*** ERROR: Missing package version (-v)." ]; then
        FAILS=$((${FAILS} + ${r}))
        echo "passed."
    else
        echo "FAILED."
    fi
fi

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distmk -r'..."
    ${LEAKCHECK_CMD} ${DISTMK} -t ${TREE} -p "test" -v "1.0"
else
    echo -n "checking 'distmk -r'..."
    out="$(${DISTMK} -t ${TREE} -p "test" -v "1.0" 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "*** ERROR: Missing package release (-r)." ]; then
        FAILS=$((${FAILS} + ${r}))
        echo "passed."
    else
        echo "FAILED."
    fi
fi

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distmk -l'..."
    ${LEAKCHECK_CMD} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1
else
    echo -n "checking 'distmk -l'..."
    out="$(${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "*** ERROR: Missing path to plist file (-l)." ]; then
        FAILS=$((${FAILS} + ${r}))
        echo "passed."
    else
        echo "FAILED."
    fi
fi

rm -rf ${TREE}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 5 failures
    [ ${FAILS} -ne 5 ] && exit 1 || exit 0
fi
