#!/bin/sh
# Make sure distmk does not remove the tmpdir.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking that distmk preserves --tmpdir..."
    ${LEAKCHECK_CMD} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    echo -n "checking that distmk preserves --tmpdir..."
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
    r=$?
    count=$(ls -1 ${TMPDIST} | wc -l)
    if [ ${r} -ne 0 -o ! -d ${TMPDIST} -o ${count} -ne 0 ]; then
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    else
        echo "passed."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected zero failures
    [ ${FAILS} -gt 0 ] && exit 1 || exit 0
fi
