#!/bin/sh
# Verify the options on distplist.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTPLIST=$(readlink -f ../distplist)
FAILS=0
TREE="$(mktemp -d)"
TMPDIR="$(mktemp -d)"

[ ! -x ${DISTPLIST} ] && ${MAKE} -C ../ distplist

if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distplist -t'..."
    ${LEAKCHECK_CMD} ${DISTPLIST}
else
    echo -n "checking 'distplist -t'..."
    out="$(${DISTPLIST} 2>&1)"
    r=$?
    if [ ${r} -eq 1 -a "${out}" = "*** ERROR: Missing package root tree (-t)." ]; then
        FAILS=$((${FAILS} + ${r}))
        echo "passed."
    else
        echo "FAILED."
    fi
fi

SHORTFILE="$(basename ${0})"
cp ${SHORTFILE} ${TREE}
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distplist -t PATH'..."
    ${LEAKCHECK_CMD} ${DISTPLIST} -t ${TREE} > ${TMPDIR}/plist
else
    echo -n "checking 'distplist -t PATH'..."
    ${DISTPLIST} -t ${TREE} > ${TMPDIR}/plist
    mode="$(stat -c %a ${TREE}/${SHORTFILE})"
    [ $(echo ${mode} | wc -c) -eq 4 ] && mode="0${mode}"
    echo "${SHORTFILE} $(id -un)($(id -u)) $(id -gn)($(id -g)) ${mode}" > ${TMPDIR}/plist.a
    cmp -s ${TMPDIR}/plist ${TMPDIR}/plist.a
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ ${r} -eq 1 ] && echo "FAILED." || echo "passed."
fi

SHORTFILE="$(basename ${0})"
cp ${SHORTFILE} ${TREE}
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distplist -t PATH -u root -g root'..."
    ${LEAKCHECK_CMD} ${DISTPLIST} -t ${TREE} -u root -g root > ${TMPDIR}/plist
else
    echo -n "checking 'distplist -t PATH -u root -g root'..."
    ${DISTPLIST} -t ${TREE} -u root -g root > ${TMPDIR}/plist
    mode="$(stat -c %a ${TREE}/${SHORTFILE})"
    [ $(echo ${mode} | wc -c) -eq 4 ] && mode="0${mode}"
    echo "${SHORTFILE} root(0) root(0) ${mode}" > ${TMPDIR}/plist.a
    cmp -s ${TMPDIR}/plist ${TMPDIR}/plist.a
    r=$?
    FAILS=$((${FAILS} + ${r}))
    [ ${r} -eq 1 ] && echo "FAILED." || echo "passed."
fi

rm -rf ${TREE}
rm -rf ${TMPDIR}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 1 failure
    [ ${FAILS} -ne 1 ] && exit 1 || exit 0
fi
