#!/bin/sh
# Verify we can remove packages

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
DISTRM=$(readlink -f ../distrm)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${DISTRM} ] && ${MAKE} -C ../ distrm

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
PKG1="test-1.0-1.$(uname -m).dist"

# create another package for test purposes
mv ${TREE}/usr/sbin/distmk ${TREE}/usr/sbin/distmk2

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
PKG2="test-2.0-1.$(uname -m).dist"

# remove a package
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm PKG'..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking 'distrm PKG'..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    lines=$(echo "${out}" | wc -l)
    out="$(echo "${out}" | grep "Removing")"
    if [ ${r} -eq 0 -a ${lines} -eq 1 -a ! -z "${out}" -a ! -d ${OUTPUT}/r/usr -a ! -d ${OUTPUT}/r/usr/sbin -a ! -x ${OUTPUT}/r/usr/sbin/distmk ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# remove a package that does not exist (expect fail)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm PKG' on non-existent package..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r thing
else
    echo -n "checking 'distrm PKG' on non-existent package..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r thing 2>&1)"
    r=$?
    lines=$(echo "${out}" | wc -l)
    if [ ${r} -eq 1 -a ${lines} -eq 1 -a "${out}" = "No installed package matching 'thing' found." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# remove a multi-version package (expect fail)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm PKG' on a multi-version package (expect fail)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking 'distrm PKG' on a multi-version package (expect fail)..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    lines=$(echo "${out}" | wc -l)
    if [ ${r} -eq 1 -a ${lines} -eq 1 -a "${out}" = "*** ERROR: More than one version of test found, please specify one to remove." ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# remove a multi-version package (expect success)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm PKG' on a multi-version package (expect success)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test-1.0
else
    echo -n "checking 'distrm PKG' on a multi-version package (expect success)..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    ${DISTADD} -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test-1.0 2>&1)"
    r=$?
    lines=$(echo "${out}" | wc -l)
    out="$(echo "${out}" | grep "Removing")"
    if [ ${r} -eq 0 -a ${lines} -eq 1 -a ! -z "${out}" -a -x ${OUTPUT}/r/usr/sbin/distmk2 -a ! -x ${OUTPUT}/r/usr/sbin/distmk ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
