#!/bin/sh
# Verify distrm can run package removal scripts

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
DISTRM=$(readlink -f ../distrm)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"
PKG4="test-4.0-1.$(uname -m).dist"

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${DISTRM} ] && ${MAKE} -C ../ distrm

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# PKG1 - has a functional preremove
cat << EOF > ${META}/preremove
#!/bin/sh
if [ "\${ACTION}" = "remove" ]; then
    echo "*** PREREMOVE RUNS ON REMOVE ***"
fi
EOF

chmod 0755 ${META}/preremove
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/preremove

# PKG2 - has a non-functional preremove
cat << EOF > ${META}/preremove
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/preremove
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "2.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/preremove

# PKG3 - has a functional postremove
cat << EOF > ${META}/postremove
#!/bin/sh
if [ "\${ACTION}" = "remove" ]; then
    echo "*** POSTREMOVE RUNS ON REMOVE ***"
fi
EOF

chmod 0755 ${META}/postremove
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "3.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/postremove

# PKG4 - has a non-functional postremove
cat << EOF > ${META}/postremove
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/postremove
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "4.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "4.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
rm -f ${META}/postremove

#############
# preremove #
#############

# check that package runs preremove script on removal
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distrm preremove script execution..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking distrm preremove script execution..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "*** PREREMOVE RUNS ON REMOVE ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# check that preremove failure halts distrm
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distrm halts on preremove script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking distrm halts on preremove script failure..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 1 -a "${out}" = "*** preremove failed for test, skipping package" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# check that preremove failure can be ignored (-F)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm -F' continues on preremove script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
    ${LEAKCHECK_CMD} ${DISTRM} -F -r ${OUTPUT}/r test
else
    echo -n "checking 'distrm -F' continues on preremove script failure..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
    out="$(${DISTRM} -F -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "*** preremove failed for test, continuing anyway" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# check that preremove failure can be ignored by skipping scripts (-S)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
     echo "checking 'distrm -S' continues on preremove script failure..."
     ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2}
     ${LEAKCHECK_CMD} ${DISTRM} -S -r ${OUTPUT}/r test
else
     echo -n "checking 'distrm -S' continues on preremove script failure..."
     ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2} >/dev/null 2>&1
     out="$(${DISTRM} -S -r ${OUTPUT}/r test 2>&1)"
     r=$?
     out="$(echo "${out}" | grep "Removing")"
     if [ ${r} -eq 0 -a ! -z "${out}" ]; then
         echo "passed."
     else
         FAILS=$((${FAILS} + 1))
         echo "FAILED."
     fi
fi

##############
# postremove #
##############

# check that package runs postremove script on removal
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distrm postremove script execution..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG3}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking distrm postremove script execution..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG3} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "*** POSTREMOVE RUNS ON REMOVE ***" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# check that postremove failure halts distrm
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking distrm halts on postremove script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG4}
    ${LEAKCHECK_CMD} ${DISTRM} -r ${OUTPUT}/r test
else
    echo -n "checking distrm halts on postremove script failure..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG4} >/dev/null 2>&1
    out="$(${DISTRM} -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 1 -a "${out}" = "*** postremove failed for test, skipping package" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# check that postremove failure can be ignored (-F)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm -F' continues on postremove script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG4}
    ${LEAKCHECK_CMD} ${DISTRM} -F -r ${OUTPUT}/r test
else
    echo -n "checking 'distrm -F' continues on postremove script failure..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG4} >/dev/null 2>&1
    out="$(${DISTRM} -F -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | tail -n 1)"
    if [ ${r} -eq 0 -a "${out}" = "*** postremove failed for test, continuing anyway" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# check that postremove failure can be ignored by skipping scripts (-S)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'distrm -S' continues on postremove script failure..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG4}
	${LEAKCHECK_CMD} ${DISTRM} -S -r ${OUTPUT}/r test
else
    echo -n "checking 'distrm -S' continues on postremove script failure..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG4} >/dev/null 2>&1
    out="$(${DISTRM} -S -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Removing")"
    if [ ${r} -eq 0 -a ! -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected 0 failures
    [ ${FAILS} -eq 0 ] && exit 0 || exit 1
fi
