#!/bin/sh
# Verify etcmerge works as expected

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
ETCMERGE=$(readlink -f ../etcmerge)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${ETCMERGE} ] && ${MAKE} -C ../ etcmerge

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
PKG="test-1.0-1.$(uname -m).dist"

# initial configuration install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcmerge -i' for initial config file install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r
else
    echo -n "checking 'etcmerge -i' for initial config file install..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -i -r ${OUTPUT}/r 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a -r ${OUTPUT}/r/etc/issue ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# initial configuration install (pkgspec)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcmerge -i PKG' for initial config file install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r test
else
    echo -n "checking 'etcmerge -i PKG' for initial config file install..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -i -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a -r ${OUTPUT}/r/etc/issue ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# query run (exit code indicates merge or not) single pkg - merge expected
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo -n "checking 'etcmerge -q -i -m PKG' for single package..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -q -i -m -r ${OUTPUT}/r test
else
    echo -n "checking 'etcmerge -q -i -m PKG' for single package..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -q -i -m -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue" | grep "simulated")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a ! -r ${OUTPUT}/r/etc/issue ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# query run (exit code indicates merge or not) all pkgs - merge expected
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcmerge -q -i -m' for system..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -q -i -m -r ${OUTPUT}/r
else
    echo -n "checking 'etcmerge -q -i -m' for system..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -q -i -m -r ${OUTPUT}/r 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue" | grep "simulated")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a ! -r ${OUTPUT}/r/etc/issue ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# query run (exit code indicates merge or not) all pkgs - no merge expected
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcmerge -q -i -m' for system (no merge expected)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -i -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -q -i -m -r ${OUTPUT}/r
else
    echo -n "checking 'etcmerge -q -i -m' for system (no merge expected)..."
    ${DISTADD} -d -i -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -q -i -m -r ${OUTPUT}/r 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue" | grep "simulated")"
    if [ ${r} -eq 1 -a -z "${out}" -a ! -r ${OUTPUT}/r/etc/issue ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# honor /etc/dist/install.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/install.list
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking that etcmerge honors /etc/dist/install.list..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r test
else
    echo -n "checking that etcmerge honors /etc/dist/install.list..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -i -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "/etc/issue already exists, overwriting")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a "$(cat ${OUTPUT}/r/etc/issue)" = "System ready" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# ignore /etc/dist/install.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/install.list
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking that 'etcmerge -I' ignores /etc/dist/install.list..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -I -i -m -q -r ${OUTPUT}/r test
else
    echo -n "checking that 'etcmerge -I' ignores /etc/dist/install.list..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -I -i -m -q -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue" | grep "simulated")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a "$(cat ${OUTPUT}/r/etc/issue)" = "other string" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# honor /etc/dist/skip.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/skip.list
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking that etcmerge honors /etc/dist/skip.list..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r test
else
    echo -n "checking that etcmerge honors /etc/dist/skip.list..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -i -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "/etc/issue")"
    if [ ${r} -eq 0 -a -z "${out}" -a "$(cat ${OUTPUT}/r/etc/issue)" = "other string" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# ignore /etc/dist/skip.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/skip.list
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking that 'etcmerge -S' ignores /etc/dist/skip.list..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -S -i -m -q -r ${OUTPUT}/r test
else
    echo -n "checking that 'etcmerge -S' ignores /etc/dist/skip.list..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    out="$(${ETCMERGE} -S -i -m -q -r ${OUTPUT}/r test 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "Installing new" | grep "/etc/issue" | grep "simulated")"
    if [ ${r} -eq 0 -a ! -z "${out}" -a "$(cat ${OUTPUT}/r/etc/issue)" = "other string" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
