#!/bin/sh
# Verify etcorphan works as expected

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

LEAKCHECK_CMD=
[ "${LEAKCHECK}" = "y" ] && LEAKCHECK_CMD="valgrind --leak-check=full --show-reachable=yes "

DISTMK=$(readlink -f ../distmk)
DISTADD=$(readlink -f ../distadd)
ETCMERGE=$(readlink -f ../etcmerge)
ETCORPHAN=$(readlink -f ../etcorphan)
TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

[ ! -x ${DISTMK} ] && ${MAKE} -C ../ distmk
[ ! -x ${DISTADD} ] && ${MAKE} -C ../ distadd
[ ! -x ${ETCMERGE} ] && ${MAKE} -C ../ etcmerge
[ ! -x ${ETCORPHAN} ] && ${MAKE} -C ../ etcorphan

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

echo "Big giant legal file." > ${META}/legal

cat << EOF > ${META}/pkginfo
desc="Test program"
maintainer="J. Random Packager <someone@example.net>"
license=WTFPL
EOF

# create the package
if [ "${LEAKCHECK}" = "y" ]; then
    ${LEAKCHECK_CMD} \
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST}
else
    ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist \
              -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} >/dev/null 2>&1
fi
PKG="test-1.0-1.$(uname -m).dist"

# report owned files (via single file specification and -r usage)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcorphan FILE' for initial config file install..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r
    ${LEAKCHECK_CMD} ${ETCORPHAN} -r ${OUTPUT}/r /etc/issue
else
    echo -n "checking 'etcorphan FILE' for initial config file install..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    ${ETCMERGE} -i -r ${OUTPUT}/r >/dev/null 2>&1
    out="$(${ETCORPHAN} -r ${OUTPUT}/r /etc/issue 2>&1)"
    r=$?
    out="$(echo "${out}" | grep "/etc/issue:" | cut -d ' ' -f 2)"
    if [ ${r} -eq 0 -a ! -z "${out}" -a "$out" = "$(basename ${PKG} .dist)" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# report unowned files (with no unowned files existing)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcorphan' for unowned file reporting (w/ no unowned files)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r
    ${LEAKCHECK_CMD} ${ETCORPHAN} -r ${OUTPUT}/r
else
    echo -n "checking 'etcorphan' for unowned file reporting (w/ no unowned files)..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    ${ETCMERGE} -i -r ${OUTPUT}/r >/dev/null 2>&1
    out="$(${ETCORPHAN} -r ${OUTPUT}/r 2>&1)"
    r=$?
    if [ ${r} -eq 0 -a -z "${out}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

# report unowned files (with one unowned files existing)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
if [ "${LEAKCHECK}" = "y" ]; then
    echo "checking 'etcorphan' for unowned file reporting (w/ one unowned file)..."
    ${LEAKCHECK_CMD} ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG}
    ${LEAKCHECK_CMD} ${ETCMERGE} -i -r ${OUTPUT}/r
    ${LEAKCHECK_CMD} ${ETCORPHAN} -r ${OUTPUT}/r
else
    echo -n "checking 'etcorphan' for unowned file reporting (w/ one unowned file)..."
    ${DISTADD} -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
    ${ETCMERGE} -i -r ${OUTPUT}/r >/dev/null 2>&1
    echo "example" > ${OUTPUT}/r/etc/example.conf
    out="$(${ETCORPHAN} -r ${OUTPUT}/r 2>&1)"
    r=$?
    if [ ${r} -eq 0 -a ! -z "${out}" -a "${out}" = "/etc/example.conf" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

if [ "${LEAKCHECK}" = "y" ]; then
    exit 0
else
    # expected $EXPECTED_FAILS failures
    [ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
fi
