#!/bin/sh
#
# clean-chroot.sh: Clean the $CHROOT path and preserve some things
#
# {{{ Apache License version 2.0
#
# Copyright 2012-2017 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $CHROOT               The chroot path (cannot be '/').
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${CHROOT}" ]; then
    echo "*** You must specify the CHROOT= variable." >&2
    exit 1
fi

if [ -d "${CHROOT}" ] && [ "$(readlink -f ${CHROOT})" = "/" ]; then
    exit 2
fi

if [ $(id -u) -ne 0 ]; then
    echo "*** You must be root to perform a chroot clean." >&2
    exit 3
fi

for subdir in bin boot etc lib lib64 mnt run root sbin tmp usr var ; do
    [ ! -d "${CHROOT}/${subdir}" ] && continue
    echo -n "Removing ${CHROOT}/${subdir}..."
    rm -rf ${CHROOT}/${subdir} || exit 4
    echo "done."
done

exit 0
