#!/bin/sh
#
# cleansum.sh: Reset the archives file to just have archive filenames
#              and include lines.
#
# {{{ Apache License version 2.0
#
# Copyright 2017 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# This script returns zero on success, non-zero on error.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${WRKDIR}" ] || [ ! -d "${WRKDIR}" ]; then
    exit 1
fi

CWD="$(pwd)"
NEW="${WRKDIR}/archives.new"

if [ -f "${CWD}/archives" ]; then
    rm -f "${NEW}"

    while read line ; do
        echo "${line}" | grep -qE "\(.+\)=" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            echo "${line}" | cut -d '(' -f 2 | cut -d ')' -f 1 >> "${NEW}"
            continue
        fi

        echo "${line}" | grep -qE "^(SHA|MD)[0-9]+\ " >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            echo "${line}" | cut -d ' ' -f 3 >> "${NEW}"
            continue
        fi

        echo "${line}" >> "${NEW}"
    done < "${CWD}/archives"

    if [ -f "${NEW}" ]; then
        cp "${NEW}" "${CWD}/archives"
    fi
fi
