#!/bin/sh
#
# hardening-check.sh: Run hardening-check script on WRKINST, report.
#
# {{{ Apache License version 2.0
#
# Copyright 2018 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $WRKINST              The staging root directory.
#    $LIBDIR               Usually /usr/lib, but you never know.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${WRKINST}" ] || [ ! -d "${WRKINST}" ]; then
    echo "${WRKINST} not found."
    exit 0
fi

PRUNEOPT=
if [ -z "${LIBDIR}" ] && [ -d "${WRKINST}${LIBDIR}" ]; then
    # Ignore these glibc-specific files.
    PRUNEOPT="-not \( -path ${WRKINST}${LIBDIR}/gconv -prune \)"
fi

find ${WRKINST}/ ${PRUNEOPT} -type f | while read fname ; do
    [ -f "$fname" ] || continue
    t="$(file -b --mime-type "$fname")"

    if [ "$t" = "application/x-executable" -o "$t" = "application/x-sharedlib" ]; then
        # Skip 32-bit content, which is likely special emulator or boot
        # loader files.
        file "$fname" | grep -q "ELF 32-bit" >/dev/null 2>&1
        [ $? -eq 0 ] && continue

        hardening-check -p -q -v "$fname"
    fi
done

exit 0
