#!/bin/sh
#
# incver.sh: Increment DISTRIBUTION_VER and reset all package REL
#            counts to 1 for packages versioned after the
#            DISTRIBUTION_VER.
#
# {{{ Apache License version 2.0
#
# Copyright 2017 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $TOPSRC               The top level project directory.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${TOPSRC}" ] || [ ! -d "${TOPSRC}" ]; then
    exit 1
fi

CWD="$(pwd)"

# Increment the DISTRIBUTION_VERSION value
v="$(grep ^DISTRIBUTION_VERSION ${TOPSRC}/mk/defs.mk | rev | cut -d ' ' -f 1 | rev)"
major="$(echo ${v} | cut -d '.' -f 1)"
minor="$(echo ${v} | cut -d '.' -f 2)"
update="$(echo ${v} | cut -d '.' -f 3)"
newminor=$((${minor} + 1))
sed -i -e "s|^DISTRIBUTION_VERSION\ =\ .*\$|DISTRIBUTION_VERSION = ${major}.${newminor}.${update}|g" ${TOPSRC}/mk/defs.mk

# Reset all REL values in packages that use DISTRIBUTION_VERSION
find ${TOPSRC}/ -type f -name Makefile | xargs grep "VER =" | grep DISTRIBUTION_VERSION | cut -d ':' -f 1 | \
while read f ; do
    sed -i -e "s|^REL\ =\ .*\$|REL = 1|g" ${f}
done
