#!/bin/sh
#
# parsebool.sh: Look at variables and figure out the boolean value.
#               Supports true/false, T/F, yes/no, oui/non, and so on.
#
# {{{ Apache License version 2.0
#
# Copyright 2005-2016 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# The following environment variables and arguments are used:
#    $1           The boolean value to test.
# The script returns 'y' or 'n' for use in the Makefiles.

bool=${1}

if [ -z "${bool}" ]; then
    echo "n"
    exit 0
fi

case $bool in
    [Yy][Ee][Ss]|Y|y|[Tt][Rr][Uu][Ee]|T|t)
        echo "y"
        ;;
    [Nn][Oo]|[Ff][Aa][Ll][Ss][Ee]|N|n|F|f)
        echo "n"
        ;;
    *)
        echo "X"
        ;;
esac

exit 0
