#!/bin/sh
#
# patch.sh: Apply patches to extracted source tree.
#
# {{{ Apache License version 2.0
#
# Copyright 2011-2017 David Cantrell <david.l.cantrell@gmail.com>
#                     Chris Lumens <chris@bangmoney.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $WRKDIR               The top level work directory.
#    $WRKSRC               The extracted source directory.
#    $PATCHLEVEL           The directory levels to trim when applying.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${WRKSRC}" ] || [ ! -d "${WRKSRC}" ]; then
    exit 0
fi

CWD="$(pwd)"

( if [ -d "${CWD}/patches" ]; then
      find ${CWD}/patches -type f -name "*.patch" | while read patchfile ; do
          d="$(head -n 2 ${patchfile} | tail -n 1 | cut -d ' ' -f 2 | cut -f 1 | cut -d '/' -f 1)"
          s="${WRKSRC}"

          patch -d ${s} -p ${PATCHLEVEL} -F 0 -b -z .orig --verbose -i ${patchfile}
      done
  fi
) 2>&1 | tee ${WRKDIR}/patch.log
