#!/bin/sh
#
# show-orphans.sh: Show packages not required by any other package in the
#                  source tree.  Useful for finding candidates for removal.
#
# {{{ Apache License version 2.0
#
# Copyright 2013-2017 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $TOPSRC               The top level distribution source directory.
#    $COLLECTION           The name of the collection.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${TOPSRC}" ] || [ ! -d "${TOPSRC}" ]; then
    exit 1
fi

if [ -z "${COLLECTION}" ] || [ ! -d "${TOPSRC}/${COLLECTION}" ]; then
    exit 2
fi

get_all_packages() {
    find ${TOPSRC}/${COLLECTION} -type f -name pkginfo | \
    while read package ; do
        pkgpath="$(dirname ${package} | sed -e "s|^${TOPSRC}/||g")"
        echo "${pkgpath}"
    done
}

# This is just a sloppy check to see if any packages require each package.
# Not all orphans are candidates for removal.
get_all_packages | while read pkgpath ; do
    find ${TOPSRC} -name Makefile | xargs grep -q "${pkgpath}" >/dev/null 2>&1
    [ ${?} -ne 0 ] && echo "${pkgpath}"
done

exit 0
