#
# Mirror targets
#
# {{{ Apache License version 2.0
#
# Copyright 2012-2016 David Cantrell <david.l.cantrell@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# }}}

-include defs.mk
-include mk/defs.mk

HELP += "    mirrors              Run all mirror-* targets\n"

mirrors: mirror-distfiles mirror-devel

# https://www.burdell.org/peachtree/distfiles/
DISTFILES_HOST = kevlar.burdell.org
DISTFILES_PATH = /srv/www/burdell.org/peachtree/distfiles/

HELP += "   mirror-distfiles     Upload distfiles to $(DISTFILES_HOST)\n"

# http://www.burdell.org/peachtree/release/devel/
DEVEL_HOST = kevlar.burdell.org
DEVEL_PATH = /srv/www/burdell.org/peachtree/releases/devel/

HELP += "   mirror-devel         Upload devel release tree to $(DEVEL_HOST)\n"

# The current stable release
RELEASE_HOST = kevlar.burdell.org
RELEASE_PATH = /srv/www/burdell.org/peachtree/releases/$(DISTRIBUTION_VERSION)/

ifeq ($(TARGET_MACH),x86_64)
RELEASE_EXCLUDE = --exclude ppc64
endif

ifeq ($(TARGET_MACH),ppc64)
RELEASE_EXCLUDE = --exclude x86_64
endif

HELP += "   pull                 Download current stable release to $(RELEASE)\n"
HELP += "   pull-devel           Download current development release to $(RELEASE)\n"

# Some options are only available in later rsync protocol versions
RSYNC30_OPTS = -rltOIvz --rsh=ssh --progress --delete
RSYNC31_OPTS = $(RSYNC30_OPTS) -J

# This will only work if the current user has 'kevlar.burdell.org' access
# configured in their ~/.ssh/config file.
#
# NOTE:  Common options are -avz, but we use these options:
#            -r    Recursive
#            -L    Symlinks are symlinks, do not dereference
#            -t    Preserve timestamps
#            -O    Omit directories when preserving timestamps (only
#                  works if you are the remote owner of the dir, group
#                  write access is not sufficient)
#            -J    Same as -O, but for symlinks (same reason)
#            -I    Ignore timestamps since those may be all over the place
#            -v    Verbose
#            -z    Compress the network traffic
mirror-distfiles:
	@if [ -d $(DISTFILES) ]; then \
		rsync $(RSYNC31_OPTS) $(DISTFILES)/ $(DISTFILES_HOST):$(DISTFILES_PATH) ; \
		if [ $$? -eq 12 ]; then \
			rsync $(RSYNC30_OPTS) $(DISTFILES)/ $(DISTFILES_HOST):$(DISTFILES_PATH) ; \
		fi ; \
	fi

mirror-devel:
	@if [ -d $(RELEASE) ]; then \
		rsync $(RSYNC31_OPTS) $(RELEASE)/ $(DEVEL_HOST):$(DEVEL_PATH) ; \
		if [ $$? -eq 12 ]; then \
			rsync $(RSYNC30_OPTS) $(RELEASE)/ $(DEVEL_HOST):$(DEVEL_PATH) ; \
		fi ; \
	fi

pull:
	[ -d $(RELEASE) ] || sudo mkdir -p $(RELEASE)
	rsync $(RSYNC31_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$(RELEASE_PATH) $(RELEASE)/ ; \
	if [ $$? -eq 12 ]; then \
		rsync $(RSYNC30_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$(RELEASE_PATH) $(RELEASE)/ ; \
	fi

# This pulls from devel, but stores in in the distribution version release
# directory.
pull-devel:
	releasepath="$$(echo $(RELEASE_PATH) | sed -e 's|$(DISTRIBUTION_VERSION)|devel|g')" ; \
	[ -d $(RELEASE) ] || sudo mkdir -p $(RELEASE) ; \
	rsync $(RSYNC31_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$${releasepath} $(RELEASE)/ ; \
	if [ $$? -eq 12 ]; then \
		rsync $(RSYNC30_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$${releasepath} $(RELEASE)/ ; \
	fi
