#!/bin/sh
#
# Verify there are no cmake module files outside of /usr/share/cmake
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
TMPDIR="$(mktemp -d)"
RET=0

rm -f $LOG

echo -n "checking for cmake files outside of /usr/share/cmake..."

find /usr -not \( -path /usr/src -prune \) \
          -not \( -path /usr/obj -prune \) \
          -not \( -path /usr/distfiles -prune \) \
          -not \( -path /usr/release -prune \) \
          -not \( -path /usr/share/doc -prune \) \
          -not \( -path /usr/share/licenses -prune \) \
          -not \( -path /usr/X11 -prune \) \
          -type f -name "*.cmake" | \
grep -v "/usr/share/cmake" > $TMPDIR/bad-cmake-files
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo "find /usr -type f -name \"*.cmake\" | grep -v \"/usr/share/cmake\"" >> $LOG
    echo >> $LOG
    while read line ; do
        echo "$line is not in /usr/share/cmake" >> $LOG
    done < $TMPDIR/bad-cmake-files
    RET=$(($RET + 1))
else
    echo "pass"
fi

echo -n "checking for cmake files outside of /usr/X11/share/cmake..."

find /usr/X11 -type f -name "*.cmake" | \
grep -v "/usr/X11/share/cmake" > $TMPDIR/bad-cmake-files
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo "find /usr -type f -name \"*.cmake\" | grep -v \"/usr/X11/share/cmake\"" >> $LOG
    echo >> $LOG
    while read line ; do
        echo "$line is not in /usr/X11/share/cmake" >> $LOG
    done < $TMPDIR/bad-cmake-files
    RET=$(($RET + 1))
else
    echo "pass"
fi

rm -rf $TMPDIR
exit $RET
